def search(thing) -> 'If thing is a string type, return the unicode number of it     If thing is a int type, return the corresponding character of it':
    if type(thing) == type(''):
        return ord(thing)
    elif type(thing) == type(1):
        return chr(thing)

def table(items, cols, add_numbers=False):
    rows = items // cols
    table = list('' for x in range(rows+1))
    char = 1
    for col in range(1, cols + 1):
        for row in range(1, rows+1):
            if add_numbers == True:
                table[row] += '{:3.0f}'.format(char) + ' '
            if add_numbers == False:
                table[row] += chr(char)
            else:
                table[row] += ' ' + chr(char)
            char += 1
    return table

def listcode(items, add_numbers=False):
    table = []
    char = 1
    for x in range(1, items + 1):
        if add_numbers == True:
            table.append('{:3.0f}'.format(char) + ' ')
        if add_numbers != True:
            table.append(chr(char))
        else:
            table[-1] += chr(char)
        char += 1
    return table



# -*- coding: utf-8 -*-

try:
    from mathematics import *
    from password import *
    from unicode import *
    from word_bank import *
    import __init__ as pyex
except (ImportError, ModuleNotFoundError) as msg:
    from pyextension.mathematics import *
    from pyextension.password import *
    from pyextension.unicode import *
    from pyextension.word_bank import *
    import pyextension as pyex
import os

def _clear():
    try:
        for root, dirs, files in os.walk('__pycache__', topdown=False):
            for name in files:
                os.remove(os.path.join(root, name))
        os.rmdir('__pycache__')
    except OSError:
        pass
    try:
        for root, dirs, files in os.walk('word_bank/__pycache__', topdown=False):
            for name in files:
                os.remove(os.path.join(root, name))
        os.rmdir('word_bank/__pycache__')
    except OSError:
        pass
    try:
        os.remove('word bank.dat')
    except OSError:
        pass
    return True

def _test_pyextension():
    pyex.computer
    pyex.computer()
    pyex.python
    pyex.python()
    pyex.opensource('pyextension')
    pyex.opensource('mathematics')
    pyex.opensource('password')
    pyex.opensource('unicode')
    pyex.opensource('word bank')
    pyex.opensource('word bank data')
    pyex.message()
    pyex.description()
    pyex.GNU_General_Public_License()
    pyex.GPL()
    pyex.license()
    pyex.word_bank()
    _clear()
    return True

def _test_math():
    return _test_mathematics()

def _test_mathematics():
    floatdd(1.23)
    fibo(5)
    fibo(found=5)
    fibo(to=5)
    arithmetic_sequence('all', '12345')
    _clear()
    return True

def _test_password():
    string = '''
_test_password :

testing undo
testing set
testing reset
testing reverse
testing caesar
testing vigenere
testing get
'''
    __test__ = Encrypt('abc')
    __test__.set('abcdefghijklmnopqrstuvwxyz')
    __test__.reset()
    __test__.undo()
    __test__.get()
    a = Encrypt(string)
    a.reverse()
    a.caesar(3)
    a.vigenere_cipher('sky')
    string1 = a.get()
    b = Decrypt(string1)
    b.vigenere_cipher('sky')
    b.caesar(3)
    b.reverse()
    string2 = b.get()
    _clear()
    if string2 != string:
        return False
    else:
        return True

def _test_unicode():
    search(1)
    search('a')
    table(64, 8)
    table(64, 8, True)
    listcode(64)
    listcode(64, True)
    return True

def _test_wordbank():
    import time
    location = '_test_wordbank--' + str(time.time()) + '.dat'
    install(location, get())
    a = Word(location)
    a.search('a')
    a.get()
    uninstall(location)
    _clear()
    return True

def _test(module='all'):
    if module == 'all':
        return (
            _test_pyextension(),
            _test_mathematics(),
            _test_password(),
            _test_unicode(),
            _test_wordbank()
            )
    elif module == 'pyextension':
        return _test_pyextension()
    elif module == 'mathematics' or module == 'math':
        return _test_mathematics()
    elif module == 'password':
        return _test_password()
    elif module == 'unicode':
        return _test_unicode()
    elif module == 'word bank' or module == 'word_bank':
        return _test_wordbank()

_clear()



# -*- coding:utf-8  -*-

WORD = {
    'a' : [
        'a',
        'ability',
        'able',
        'about',
        'above',
        'abroad',
        'absolutely',
        'accept',
        'acceptable',
        'accident',
        'according to',
        'account',
        'accuse',
        'achieve',
        'acid',
        'across',
        'act',
        'action',
        'active',
        'activity',
        'actor',
        'actress',
        'actual',
        'actually',
        'ad',
        'add',
        'address',
        'admire',
        'admit',
        'adult',
        'advantage',
        'adventure',
        'advertisement',
        'advice',
        'advise',
        'affect',
        'afford',
        'afraid',
        'after',
        'afternoon',
        'afterwards',
        'again',
        'against',
        'age',
        'aged',
        'ago',
        'agree',
        'agreement',
        'ahead',
        'aim',
        'air',
        'aircraft',
        'airport',
        'alcohol',
        'alcoholic',
        'alive',
        'all',
        'allow',
        'all right',
        'almost',
        'alone',
        'along',
        'alphabet',
        'already',
        'also',
        'although',
        'always',
        'among',
        'amount',
        'amuse',
        'amusing',
        'an',
        'ancient',
        'anger',
        'angle',
        'angry',
        'animal',
        'announce',
        'annoy',
        'annoyed',
        'annoying',
        'another',
        'answer',
        'any',
        'anybody',
        'anyone',
        'anything',
        'anyway',
        'apart',
        'apartment',
        'appear',
        'appearance',
        'apple',
        'application',
        'apply',
        'appropriate',
        'approval',
        'approve',
        'April',
        'area',
        'argue',
        'argument',
        'arm',
        'army',
        'around',
        'arrange',
        'arrangement',
        'arrest',
        'arrive',
        'arrow',
        'art',
        'article',
        'artificial',
        'artist',
        'artistic',
        'as',
        'ashamed',
        'ask',
        'asleep',
        'at',
        'atmosphere',
        'attack',
        'attention',
        'attract',
        'attractive',
        'August',
        'aunt',
        'autumn',
        'available',
        'avoid',
        'awake',
        'aware',
        'away',
        'awesome',
        'awful',
        'axe'
        ],
    'b' : [
        'baby',
        'back',
        'backwards',
        'bad',
        'badly',
        'badminton',
        'bad-tempered',
        'bag',
        'bake',
        'balance',
        'ball',
        'balloon',
        'band',
        'bank',
        'bar',
        'base',
        'baseball',
        'basic',
        'basis',
        'basket',
        'basketball',
        'bat',
        'bath',
        'bathroom',
        'be',
        'beach',
        'bear',
        'beard',
        'beat',
        'beautiful',
        'beauty',
        'become',
        'bed',
        'bedroom',
        'bee',
        'beef',
        'been',
        'beer',
        'before',
        'begin',
        'beginning',
        'behave',
        'behaviour',
        'behind',
        'belief',
        'believe',
        'bell',
        'belong',
        'below',
        'belt',
        'bend',
        'beneath',
        'benefit',
        'best',
        'bet',
        'better',
        'between',
        'beyond',
        'bicycle',
        'big',
        'bill',
        'billion',
        'bin',
        'bind',
        'biology',
        'bird',
        'birth',
        'birthday',
        'biscuit',
        'bit',
        'bite',
        'bitter',
        'black',
        'blame',
        'block',
        'blood',
        'blow',
        'blue',
        'board',
        'boat',
        'body',
        'boil',
        'bold',
        'bomb',
        'bone',
        'book',
        'boot',
        'border',
        'bored',
        'boring',
        'born',
        'both',
        'bottle',
        'bottom',
        'bow',
        'bowl',
        'box',
        'boy',
        'bra',
        'brain',
        'branch',
        'brave',
        'bread',
        'break',
        'breakfast',
        'breast',
        'breath',
        'breathe',
        'brick',
        'brief',
        'bright',
        'bring',
        'broken',
        'brother',
        'brown',
        'brush',
        'bubble',
        'build',
        'building',
        'bullet',
        'burn'
        ]
    }

def get():
    return WORD

import sys
if sys.version_info[0] == 2:
    import tkFileDialog as tk
elif sys.version_info[0] == 3 and sys.version_info[1] < 4:
    import filedialog as tk
elif sys.version_info[0] == 3 and sys.version_info[1] >= 4:
    import tkinter.filedialog as tk
else:
    raise SystemExit('This Python version is not supported by the current version of Python-Extension.')
import pickle
#try:
#    import pyextension.word_bank.word_bank_value as WORDBANK
#except:
#    import word_bank_value as WORDBANK

def install(location, obj):
    file = open(location, 'wb')
    pickle.dump(obj, file)
    file.close()

import os

def uninstall(location):
    os.remove(location)

try:
    open('word bank.dat', 'rb')
except OSError:
    install('word bank.dat', WORD)

class Word():
    def __init__(self, location=None):
        if location == None:
            location = 'word bank.dat'
        self.location = location
        file = open(location, 'rb')
        self.dic = pickle.load(file)
        file.close()
    def search(self, key):
        try:
            return self.dic[key]
        except KeyError:
            result = []
            for x in self.dic[key[0]]:
                if key in x:
                    result.append(x)
            return result
    def get(self):
        return self.dic
    def insert(self, key, item=[], save=False):
        for x in item:
            self.dic[key] = self.dic[key].append(x)
        if save == True:
            file = open(self.location, 'wb')
            pickle.dump(self.dic, file)
            file.close()
        return self.dic
    def change(self, key, item, save=False):
        if type(item) == type([]):
            self.dic[key] = item
        else:
            raise TypeError('Argument \'item\' must be list type')
        if save == True:
            file = open(self.location, 'wb')
            pickle.dump(self.dic, file)
            file.close()
        return self.dic
    def delete(self, key, item, save=False):
        if type(item) == type(1):
            del self.dic[key][item]
        elif type(item) == type(''):
            times = 0
            try:
                while True:
                    if self.dic[key][times] == item:
                        del self.dic[key][times]
                        break
                    times += 1
            except IndexError:
                raise ValueError('No item \'%s\' in word bank.dat[%s]' % (item, key))
            return dic
        elif type(item) == type([]):
            for x in item:
                if type(x) == type(0):
                    del self.dic[key][x]
                elif type(x) == type(''):
                    times = 0
                    try:
                        while True:
                            if self.dic[key][times] == x:
                                del self.dic[key][times]
                                break
                            times += 1
                    except IndexError:
                        raise ValueError('No item \'%s\' in word bank.dat[%s]' % (x, key))
        else:
            raise TypeError('type \'item\' must be string or int')
        # Delete
        if save == True:
            file = open(self.location, 'wb')
            pickle.dump(self.dic, file)
            file.close()
        return self.dic
    def save(self):
        file = open(self.location, 'wb')
        pickle.dump(self.dic, file)
        file.close()
    def saveas(self, location=None):
        if location[-4] + location[-3] + location[-2] + location[-1] != '.dat':
            location = location + '.dat'
            return 'Warning : Your location is not a dat file, we turned it to a dat file, please check the file if it not the shape you want.'
        try:
            file = open(location, 'wb')
            pickle.dump(self.dic, file)
            file.close()
        except OSError:
            pass



import pickle

WORD = {
    'a' : [
        'a',
        'ability',
        'able',
        'about',
        'above',
        'abroad',
        'absolutely',
        'accept',
        'acceptable',
        'accident',
        'according to',
        'account',
        'accuse',
        'achieve',
        'acid',
        'across',
        'act',
        'action',
        'active',
        'activity',
        'actor',
        'actress',
        'actual',
        'actually',
        'ad',
        'add',
        'address',
        'admire',
        'admit',
        'adult',
        'advantage',
        'adventure',
        'advertisement',
        'advice',
        'advise',
        'affect',
        'afford',
        'afraid',
        'after',
        'afternoon',
        'afterwards',
        'again',
        'against',
        'age',
        'aged',
        'ago',
        'agree',
        'agreement',
        'ahead',
        'aim',
        'air',
        'aircraft',
        'airport',
        'alcohol',
        'alcoholic',
        'alive',
        'all',
        'allow',
        'all right',
        'almost',
        'alone',
        'along',
        'alphabet',
        'already',
        'also',
        'although',
        'always',
        'among',
        'amount',
        'amuse',
        'amusing',
        'an',
        'ancient',
        'anger',
        'angle',
        'angry',
        'animal',
        'announce',
        'annoy',
        'annoyed',
        'annoying',
        'another',
        'answer',
        'any',
        'anybody',
        'anyone',
        'anything',
        'anyway',
        'apart',
        'apartment',
        'appear',
        'appearance',
        'apple',
        'application',
        'apply',
        'appropriate',
        'approval',
        'approve',
        'April',
        'area',
        'argue',
        'argument',
        'arm',
        'army',
        'around',
        'arrange',
        'arrangement',
        'arrest',
        'arrive',
        'arrow',
        'art',
        'article',
        'artificial',
        'artist',
        'artistic',
        'as',
        'ashamed',
        'ask',
        'asleep',
        'at',
        'atmosphere',
        'attack',
        'attention',
        'attract',
        'attractive',
        'August',
        'aunt',
        'autumn',
        'available',
        'avoid',
        'awake',
        'aware',
        'away',
        'awesome',
        'awful',
        'axe'
        ],
    'b' : [
        'baby',
        'back',
        'backwards',
        'bad',
        'badly',
        'badminton',
        'bad-tempered',
        'bag',
        'bake',
        'balance',
        'ball',
        'balloon',
        'band',
        'bank',
        'bar',
        'base',
        'baseball',
        'basic',
        'basis',
        'basket',
        'basketball',
        'bat',
        'bath',
        'bathroom',
        'be',
        'beach',
        'bear',
        'beard',
        'beat',
        'beautiful',
        'beauty',
        'become',
        'bed',
        'bedroom',
        'bee',
        'beef',
        'been',
        'beer',
        'before',
        'begin',
        'beginning',
        'behave',
        'behaviour',
        'behind',
        'belief',
        'believe',
        'bell',
        'belong',
        'below',
        'belt',
        'bend',
        'beneath',
        'benefit',
        'best',
        'bet',
        'better',
        'between',
        'beyond',
        'bicycle',
        'big',
        'bill',
        'billion',
        'bin',
        'bind',
        'biology',
        'bird',
        'birth',
        'birthday',
        'biscuit',
        'bit',
        'bite',
        'bitter',
        'black',
        'blame',
        'block',
        'blood',
        'blow',
        'blue',
        'board',
        'boat',
        'body',
        'boil',
        'bold',
        'bomb',
        'bone',
        'book',
        'boot',
        'border',
        'bored',
        'boring',
        'born',
        'both',
        'bottle',
        'bottom',
        'bow',
        'bowl',
        'box',
        'boy',
        'bra',
        'brain',
        'branch',
        'brave',
        'bread',
        'break',
        'breakfast',
        'breast',
        'breath',
        'breathe',
        'brick',
        'brief',
        'bright',
        'bring',
        'broken',
        'brother',
        'brown',
        'brush',
        'bubble',
        'build',
        'building',
        'bullet',
        'burn'
        ]
    }

'bra'
'breast'

def get():
    return WORD



# -*- coding: utf-8  -*-

from pyextension import *

def opensource(module='tkextension'):
    if module == 'tkextension' or module == '__init__':
        file = open('__init__.py')
        get = file.read()
        file.close()
        return get
    elif module == 'blackboard index' or module == 'blackboard_index':
        file = open('blackboard.py')
        get = file.read()
        file.close()
        return get
    elif module == 'test':
        file = open('test.py')
        get = file.read()
        file.close()
        return get
    elif module == 'timer index' or module == 'timer_index':
        file = open('timer.py')
        get = file.read()
        file.close()
        return get
    elif module == 'tix':
        file = open('tix/__init__.py')
        get = file.read()
        file.close()
        return get
    elif module == 'tix.blackboard' or module == 'blackboard':
        file = open('tix/blackboard.py')
        get = file.read()
        file.close()
        return get
    elif module == 'tix.dialog' or module == 'dialog':
        file = open('tix/dialog.py')
        get = file.read()
        file.close()
        return get
    elif module == 'tix.filedialog' or module == 'filedialog':
        file = open('tix/filedialog.py')
        get = file.read()
        file.close()
        return get
    elif module == 'tix.timer' or module == 'timer':
        file = open('tix/timer.py')
        get = file.read()
        file.close()
        return get
    elif module == 'turtledrawer':
        file = open('turtledrawer.py')
        get = file.read()
        file.close()
        return get
    elif module == 'system':
        file = open('system.py')
        get = file.read()
        file.close()
        return get
    else:
        raise AttributeError('\'opensource\' object has no attribute \'%s\'' % module)



# -*- coding: utf-8  -*-

dic = ''

def set_dictionary(dic_type='normal', extra_arg=None):
    global dic
    if dic_type == 'normal':
        dic = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890`~!@#$%^&*()-=_+|;:\'\",./?<>[]{} \n'
    elif dic_type == 'code':
        import __init__
        import unicode
        __init__.clear()
        for x in unicode.listcode(extra_arg):
            dic += x
    elif dic_type == 'else':
        dic = extra_arg
    else:
        raise KeyError('%s type not found' % dic_type)

def get_dictionary():
    return dic

set_dictionary()

class Encrypt():
    def __init__(self, original, undo=True):
        self.original = original
        self.ciphertext = original
        self.isundo = undo
        if self.isundo == True:
            self.done = [self.original]
    def get(self):
        return self.ciphertext
    def undo(self):
        if self.isundo == True:
            try:
                self.ciphertext = self.done[-2]
                del self.done[-2]
                if self.isundo == True:
                    self.done.append(self.ciphertext)
            except IndexError:
                pass
    def reset(self):
        self.ciphertext = self.original
        if self.isundo == True:
            self.done.append(self.ciphertext)
    def set(self, value):
        self.ciphertext = value
        if self.isundo == True:
            self.done.append(self.ciphertext)
    def reverse(self):
        self.ciphertext = self.ciphertext[::-1]
        if self.isundo == True:
            self.done.append(self.ciphertext)
    def caesar_cipher(self, key, dic=dic):
        result = ''
        for x in self.ciphertext:
            index = dic.find(x)
            if index == -1:
                result += x
            else:
                index += key
                index %= len(dic)
                result += dic[index]
        self.ciphertext = result
        if self.isundo == True:
            self.done.append(self.ciphertext)
    def caesar(self, key, dic=dic) -> 'Caesar Cipher':
        self.caesar_cipher(key, dic)
    def vigenere_cipher(self, key, dic=dic):
        result = ''
        keys = []
        for x in key:
            if dic.find(x) != -1:
                keys.append(dic.find(x))
            else:
                raise KeyError('Key \'%s\' not found' % x)
        key_index = 0
        for x in self.ciphertext:
            index = dic.find(x)
            if index == -1:
                result += x
            else:
                index += keys[key_index]
                index %= len(dic)
                result += dic[index]
            key_index += 1
            key_index %= len(keys)
        self.ciphertext = result
        if self.isundo == True:
            self.done.append(self.ciphertext)
    def vigenere(self, key, dic=dic) -> 'Vigenere Cipher encrypt':
        self.vigenere_cipher(key, dic)
    def method(self, key, dic=dic):
        pass

class Decrypt():
    def __init__(self, ciphertext, undo=True):
        self.ciphertext = ciphertext
        self.original = ciphertext
        self.isundo = undo
        if self.isundo == True:
            self.done = [self.ciphertext]
    def get(self):
        return self.original
    def undo(self):
        if self.isundo == True:
            try:
                self.original = self.done[-2]
                del self.done[-2]
                if self.isundo == True:
                    self.done.append(self.original)
            except IndexError:
                pass
    def reset(self):
        self.original = self.ciphertext
        if self.isundo == True:
            self.done.append(self.original)
    def set(self, value):
        self.original = value
        if self.isundo == True:
            self.done.append(self.original)
    def reverse(self):
        self.original = self.original[::-1]
        if self.isundo == True:
            self.done.append(self.original)
    def caesar_cipher(self, key, dic=dic):
        result = ''
        for x in self.original:
            index = dic.find(x)
            if index == -1:
                result += x
            else:
                index -= key
                index %= len(dic)
                result += dic[index]
        self.original = result
        if self.isundo == True:
            self.done.append(self.original)
    def caesar(self, key, dic=dic) -> 'Caesar Cipher decrypt':
        self.caesar_cipher(key, dic)
    def vigenere_cipher(self, key, dic=dic):
        result = ''
        keys = []
        for x in key:
            if dic.find(x) != -1:
                keys.append(dic.find(x))
            else:
                raise KeyError('Key \'%s\' not found' % x)
        key_index = 0
        for x in self.original:
            index = dic.find(x)
            if index == -1:
                result += x
            else:
                index -= keys[key_index]
                index %= len(dic)
                result += dic[index]
            key_index += 1
            key_index %= len(keys)
        self.original = result
        if self.isundo == True:
            self.done.append(self.original)
    def vigenere(self, key, dic=dic) -> 'Vigenere Cipher decrypt':
        self.vigenere_cipher(key, dic)



# -*- coding: utf-8  -*-

from tkinter import *
import tkextension as tix
import platform as p
import getpass
import os

class FileTree(tix.Layout):
    def __init__(self, tkinter_attribute, filetypes=[('All Files', '')], path=None, width=200, height=200):
        self.filetypes = filetypes
        self.location = ''
        if p.system() == 'Windows':
            self.done = ['C:\\']
        elif p.system() == 'Darwin':
            self.done = ['/Users/' + getpass.getuser()]
        elif p.system() == 'Linux':
            self.done = ['/home']
        self.redo = []
        # Value
        self.tk = tkinter_attribute
        self.frame = Frame(self.tk, width=width, height=height)
        # Tkinter
        self.tree = Listbox(self.frame)
        self.tree.place(relx=0.5, rely=0.5, relwidth=0.96, relheight=0.8, anchor='center')
        self.entry = Entry(self.frame)
        self.entry.place(relx=0.5, rely=0.05, relwidth=0.96, relheight=0.1, anchor='center')
        self.undo_btn = Button(self.frame, text='Undo', command=self.undo)
        self.undo_btn.place(relx=0.15, rely=0.95, relwidth=0.3, relheight=0.1, anchor='center')
        lst = []
        for x in filetypes:
            lst.append([])
            if type(x[0]) == type(()):
                lst[-1].append([])
                for y in x[0]:
                    lst[-1][-1].append(y)
            elif type(x[0]) == type('abc'):
                lst.append([x[0]])
            if type(x[1]) == type(()):
                lst[-1].append([])
                for y in x[1]:
                    lst[-1][-1].append(y)
            elif type(x[1]) == type('abc'):
                lst.append([x[1]])
        Spinbox(self.frame, value=lst, wrap=True, state='readonly').place(relx=0.65, rely=0.95, relwidth=0.7, relheight=0.1, anchor='center')
        # Window
        if p.system() == 'Windows':
            if path == None:
                self.location = 'C:\\'
            else:
                self.location = path
        elif p.system() == 'Darwin':
            if path == None:
                self.location = '/Users/' + getpass.getuser()
            else:
                self.location = path
        elif p.system() == 'Linux':
            if path == None:
                self.location = '/home'
            else:
                self.location = path
        else:
            raise SystemExit('System \'%s\' is not supported' % p.system())
        self.entry.insert('end', self.location)
        self.items = os.listdir(self.location)
        # Items
        self.entry.bind('<Any-KeyRelease>', self.relist)
        self.tree.bind('<Double-Button-1>', self.into)
        # Bind
        self.update()
    def relist(self, event=None, mode='entry'):
        if mode == 'entry':
            self.location = self.entry.get()
            self.done.append(self.location)
            print(self.done)
        try:
            self.items = os.listdir(self.location)
            if mode == 'into':
                self.entry.delete(0, 'end')
                self.entry.insert('end', self.location)
        except OSError:
            self.entry.delete(0, 'end')
            self.entry.insert('end', self.location)
        self.update()
    def into(self, event=None):
        try:
            os.listdir(self.location)
            if p.system() == 'Windows':
                self.location = self.location + '\\' + self.tree.get('active')
            elif p.system() == 'Darwin' or p.system() == 'Linux':
                self.location = self.location + '/' + self.tree.get('active')
        except OSError:
            pass
        self.relist(mode='into')
    def update(self):
        types = []
        for x in self.filetypes:
            if type(x[1]) == type(()):
                for y in x[1]:
                    types.append(y)
            elif type(x[1]) == type('abc'):
                types.append(x[1])
        self.tree.delete(0, 'end')
        for x in self.items:
            if '' in types or '.*' in types:
                self.tree.insert('end', x)
                continue
            for y in types:
                try:
                    try:
                        open(self.location + '/' + x[:-4] + y)
                    except IndexError:
                        open('Raise Error.raise_error')
                    self.tree.insert('end', x)
                    break
                except OSError:
                    try:
                        os.listdir(self.location + '/' + x)
                        self.tree.insert('end', x)
                        break
                    except OSError:
                        pass
    def undo(self):
        self.entry.delete(0, 'end')
        try:
            self.entry.insert('end', self.done[-1])
        except IndexError:
            if p.system() == 'Windows':
                self.done = ['C:\\']
            elif p.system() == 'Darwin':
                self.done = ['/Users/' + getpass.getuser()]
            elif p.system() == 'Linux':
                self.done = ['/home']
        self.location = self.entry.get()
        del self.done[-1]
        self.done.append(self.location)
        self.relist(mode='undo')
        
    def get(self):
        self.location = self.entry.get()
        try:
            data = os.listdir(self.location)
        except OSError:
            data = self.location
        return data



# -*- coding:utf-8  -*-

from turtle import *

class Draw():
    def __init__(self, pen_attribute=None):
        if pen_attribute == None:
            self.pen = Pen()
        else:
            self.pen = pen_attribute
    def get(self):
        return self.pen
    def create_triangle(self, distance, angle=90, fill=None):
        self.pen.setheading(angle)
        # Angle
        if fill != None:
            self.pen.fillcolor(fill)
            self.pen.begin_fill()
        self.pen.left(30)
        for x in range(0, 3):
            self.pen.right(120)
            self.pen.forward(distance)
        self.pen.left(30)
        if fill != None:
            self.pen.end_fill()
        # Draw
    def create_rectangle(self, distance_x, distance_y, angle=90, fill=None):
        self.pen.setheading(angle)
        # Angle
        if fill != None:
            self.pen.fillcolor(fill)
            self.pen.begin_fill()
        for x in range(0, 2):
            self.pen.forward(distance_x)
            self.pen.right(90)
            self.pen.forward(distance_y)
            self.pen.right(90)
        if fill != None:
            self.pen.end_fill()
        # Draw
    def create_pentagon(self, distance, angle=90, fill=None):
        self.pen.setheading(angle)
        # Angle
        if fill != None:
            self.pen.fillcolor(fill)
            self.pen.begin_fill()
        for x in range(0, 5):
            self.pen.forward(distance)
            self.pen.right(72)
        if fill != None:
            self.pen.end_fill()
        # Draw
    def create_polygon(self, side, distance, angle=90, fill=None):
        internal_angle_sum = (side - 2) * 180
        # Value
        self.pen.setheading(angle)
        # Angle
        if fill != None:
            self.pen.fillcolor(fill)
            self.pen.begin_fill()
        for x in range(0, side):
            self.pen.forward(distance)
            self.pen.right(180 - internal_angle_sum / side)
        if fill != None:
            self.pen.end_fill()
        # Draw
    def create_pg(self, side, thickness, size=1, angle=90, fill=None) -> 'create psychedelic graphics':
        internal_angle_sum = (side - 2) * 180
        # Value
        self.pen.setheading(angle)
        # Angle
        pencolor = self.pen.pencolor()
        if fill != None:
            self.pen.pencolor(fill)
        for x in range(thickness * side):
            self.pen.forward(x * size)
            self.pen.right(180 - internal_angle_sum / side)
        self.pen.pencolor(pencolor)
        # Draw
    def create_pg1(self, side, thickness, size=1, rotation=1, angle=90, fill=None) -> '''create psychedelic graphics 1
The fewer sides, the more effective''':
        internal_angle_sum = (side - 2) * 180
        # Value
        self.pen.setheading(angle)
        # Angle
        pencolor = self.pen.pencolor()
        if fill != None:
            self.pen.pencolor(fill)
        for x in range(thickness * side):
            self.pen.forward(x * size)
            self.pen.right((180 - internal_angle_sum / side) + rotation)
        self.pen.pencolor(pencolor)
        # Draw
    def create_koch(self, order, size=1, angle=90, fill=None) -> 'koch curve':
        self.pen.setheading(angle)
        # Angle
        pencolor = self.pen.pencolor()
        if fill != None:
            self.pen.pencolor(fill)
        if order == 0:
            self.pen.forward(size)
        else:
            self.create_koch_curve(order - 1, size=size)
            self.pen.left(60)
            self.create_koch_curve(order - 1, size=size)
            self.pen.right(120)
            self.create_koch_curve(order - 1, size=size)
            self.pen.left(60)
            self.create_koch_curve(order - 1, size=size)
        self.pen.pencolor(pencolor)
        # Draw
    def create_koch_curve(self, order, size) -> 'koch curve service':
        if order == 0:
            self.pen.forward(size)
        else:
            self.create_koch_curve(order - 1, size=size)
            self.pen.left(60)
            self.create_koch_curve(order - 1, size=size)
            self.pen.right(120)
            self.create_koch_curve(order - 1, size=size)
            self.pen.left(60)
            self.create_koch_curve(order - 1, size=size)
    def create_koch_snowflake(self, order, size=1, angle=90, fill=None) -> 'koch snowflake':
        pencolor = self.pen.pencolor()
        if fill != None:
            self.pen.pencolor(fill)
        for x in range(3):
            self.create_koch(order, size, angle, fill)
            angle -= 120
        self.pen.pencolor(pencolor)



# -*- coding: utf-8  -*-

import os

def clear():
    try:
        for root, dirs, files in os.walk('__pycache__', topdown=False):
            for name in files:
                os.remove(os.path.join(root, name))
        os.rmdir('__pycache__')
    except OSError:
        pass
    try:
        for root, dirs, files in os.walk('tix/__pycache__', topdown=False):
            for name in files:
                os.remove(os.path.join(root, name))
        os.rmdir('tix/__pycache__')
    except OSError:
        pass
    try:
        os.remove('word bank.dat')
    except OSError:
        pass
    return True

import sys
if sys.version_info[0] != 3 or sys.version_info[1] < 4:
    raise SystemExit('This module needs Python 3.4 or more later')

from tkinter import *
from tkinter.messagebox import *
from tkinter.filedialog import *
import pickle
import time

class Pack():
    def pack(self, **options):
        return self.frame.pack(**options)
    def pack_forget(self):
        return self.frame.pack_forget()
    def pack_info(self):
        return self.frame.pack_info(self)

class Grid():
    def grid(self, **options):
        return self.frame.grid(**options)
    def grid_forget(self):
        return self.frame.grid_forget()
    def grid_info(self):
        return self.frame.pack_info(self)
    def grid_bbox(self, column=None, row=None, col2=None, row2=None):
        return self.frame.grid_bbox(column, row, col2, row2)
    def grid_columnconfigure(self, index, **options):
        return self.frame.grid_columnconfigure(index, **options)
    def grid_rowconfigure(self, index, **options):
        return self.frame.grid_rowconfigure(index, **options)
    def grid_location(self, x, y):
        return self.frame.grid_location(x, y)
    def grid_remove(self):
        return self.frame.grid_remove()
    def grid_size(self):
        return self.frame.grid_size()

class Place():
    def place(self, **options):
        return self.frame.place(**options)
    def place_forget(self):
        return self.frame.forget()
    def place_info(self):
        return self.frame.pack_info(self)

class Layout(Pack, Grid, Place):
    def forget(self):
        return self.frame.forget()

class AskValue(Layout):
    def __init__(self, tkinter_attribute, msg='', args=(0, 100), width=100, height=50):
        self.tk = tkinter_attribute
        self.frame = Frame(self.tk, width=width, height=height)
        # Tkinter
        if msg != '':
            self.label = Label(self.frame, text=msg)
            self.label.place(relx=0.5, rely=0.1, anchor='center')
        if type(args[0]) == type(0):
            self.sb = Scale(self.frame, from_=args[0], to=args[1], orient='horizontal')
        elif type(args[0]) == type('0'):
            self.sb = Spinbox(self.frame, values=args, wrap=True)
        else:
            raise TypeError('%s\'s items must be int or string')
        if msg != '':
            self.sb.place(relx=0.5, rely=0.55, relwidth=1, relheight=0.9, anchor='center')
        else:
            self.sb.place(relx=0.5, rely=0.5, relwidth=1, relheight=1, anchor='center')
        # Window
    def config(self, msg=''):
        if msg != '':
            self.label = Label(self.frame, text=msg)
            self.label.place(relx=0.5, rely=0.1, anchor='center')
            self.sb.forget()
            self.sb.place(relx=0.5, rely=0.55, relwidth=1, relheight=0.9, anchor='center')
        else:
            self.label.forget()
    def get(self):
        return self.sb.get()

class AskItem(Layout):
    def __init__(self, tkinter_attribute, msg='', items=['a', 'b', 'c'], number=1, normal=0, width=100, height=100):
        self.items = items
        self.number = number
        self.normal = normal
        # Value
        self.tk = tkinter_attribute
        self.frame = Frame(self.tk, width=width, height=height)
        # Tkinter
        if msg != '':
            self.label = Label(self.frame, text=msg)
            self.label.place(relx=0.5, rely=0.05, relwidth=1, relheight=0.1, anchor='center')
        if width != None and height != None:
            self.lb = Listbox(self.frame, width=width, height=height)
        elif width != None:
            self.lb = Listbox(self.frame, width=width)
        elif height != None:
            self.lb = Listbox(self.frame, height=height)
        else:
            self.lb = Listbox(self.frame)
        if msg != '':
            self.lb.place(relx=0.5, rely=0.55, relwidth=1, relheight=0.9, anchor='center')
        else:
            self.lb.place(relx=0.5, rely=0.5, relwidth=1, relheight=1, anchor='center')
        self.lb.selection_set(normal)
        # Window
        for x in range(0, len(items)):
            self.lb.insert('end', items[x])
        # Load
    def config(self, msg=''):
        if msg != '':
            self.label = Label(self.frame, text=msg)
            self.label.place(relx=0.5, rely=0.05, relwidth=1, relheight=0.1, anchor='center')
            self.sb.forget()
            self.sb.place(relx=0.5, rely=0.55, relwidth=1, relheight=0.9, anchor='center')
        else:
            self.label.forget()
    def get(self):
        finish = False
        get = []
        for x in range(0, len(self.items)):
            if self.lb.selection_includes(x) == 1:
                get.append(x)
        # Get
        if len(get) == self.number:
            finish = True
        else:
            get.append(self.normal)
        return get

class ControlBoard(Layout):
    def __init__(self, tkinter_attribute, msg='', item=[('scale', '', 1, 100), ('listbox', '', [1, 2, 3]), ('radiobutton', '', '1', 'IntVar()', 1)], width=100, height=100):
        self.item = item
        # Value
        self.tk = tkinter_attribute
        self.frame = Frame(self.tk, width=width, height=height)
        # Tkinter
        self.text = Text(self.frame, width=width, height=height)
        self.text.place(relx=0.5, rely=0.5, relwidth=1, relheight=1, anchor='center')
        # Text
        if msg != '':
            self.label = Label(self.frame, text=msg)
            self.text.window_create('end', window=self.label)
            self.text.insert('end', '\n')
        # Label
        self.window = []
        self.window_list = []
        self.value_list = []
        for x in item:
            if x[0] == 'scale':
                self.window.append(Scale(self.frame, from_=x[2], to=x[3], orient='horizontal'))
                self.window_list.append('scale')
            elif x[0] == 'listbox':
                self.window.append(Listbox(self.frame))
                self.window_list.append('listbox')
                for y in x[2]:
                    self.window[-1].insert('end', y)
            elif x[0] == 'radiobutton':
                self.value_list.append(x[3])
                self.window.append(Radiobutton(self.frame, text=x[2], variable=x[3], value=x[4]))
                self.window_list.append('radiobutton')
        for x in range(0, len(item)):
            if item[x][1] != '':
                self.text.insert('end', item[x][1])
            self.text.window_create('end', window=self.window[x])
            self.text.insert('end', '\n')
        # Scale & Listbox
        self.text.config(state='disabled')
        # Window
    def config(self, msg=''):
        if msg != '':
            self.label = Label(self.frame, text=msg)
            self.text.window_create('1.0', window=self.label)
            self.text.insert('1.1', '\n')
        else:
            self.label.forget()
    def get(self):
        result = []
        for x in (0, len(self.window) - 1):
            if self.window_list[x] == 'scale':
                result.append(self.window[x].get())
            elif self.window_list[x] == 'listbox':
                for y in range(0, len(self.item[x][2])):
                    if self.window[x].selection_includes(y) == 1:
                        result.append(self.item[x][2][y])
                        break
                if len(result) < x + 1:
                    result.append(self.item[x][2][0])
            elif self.window_list[x] == 'radiobutton':
                result.append(self.item[x][3].get())
            else:
                raise SystemExit
        return result

class SingleChoices(Layout):
    def __init__(self, tkinter_attribute, msg='', msg_variable=None, args=(('a', 'b', 'c'), ('e', 'f', 'g')), width=100, height=100):
        self.tk = tkinter_attribute
        self.frame = Frame(self.tk, width=width, height=height)
        # Tkinter
        self.text = Text(self.frame, width=width, height=height)
        self.text.place(relx=0.5, rely=0.5, relwidth=1, relheight=1, anchor='center')
        # Text
        if msg != '' or msg_variable != None:
            if msg_variable != None:
                self.label = Label(self.frame, variable=msg_variable)
            else:
                self.label = Label(self.frame, text=msg)
            self.text.window_create('end', window=self.label)
            self.text.insert('end', '\n')
        # Label
        self.value_list = []
        lf_list = []
        times = 0
        for x in args:
            if type(x[0]) == type(1):
                self.value_list.append(IntVar())
            else:
                self.value_list.append(StringVar())
            # Value
            lf_list.append(LabelFrame(self.frame, text=str(times + 1)))
            self.text.window_create('end', window=lf_list[-1])
            self.text.insert('end', '\n')
            # LabelFrame
            for y in x:
                Radiobutton(lf_list[times], text=y, variable=self.value_list[times], value=y).pack(side='left')
            # Tkinter
            times += 1
        # Checkbutton
        self.text.config(state='disabled')
        # Window
    def config(self, msg=''):
        if msg != '':
            self.label = Label(self.frame, text=msg)
            self.text.window_create('1.0', window=self.label)
            self.text.insert('1.1', '\n')
        else:
            self.label.forget()
    def get(self):
        get = []
        for x in self.value_list:
            get.append(x.get())
        return get

class AnswerSheet(Layout):
    def __init__(self, tkinter_attribute, msg='', msg_variable=None, args=[('a', 'b', 'c')], answer=['a'], points=[1], width=100, height=100):
        self.answer = answer
        self.points = points
        # Value
        self.tk = tkinter_attribute
        self.frame = Frame(self.tk, width=width, height=height)
        # Tkinter
        self.text = Text(self.frame, width=width, height=height)
        self.text.place(relx=0.5, rely=0.5, relwidth=1, relheight=1, anchor='center')
        # Text
        if msg != '' or msg_variable != None:
            if msg_variable != None:
                self.label = Label(self.frame, variable=msg_variable)
            else:
                self.label = Label(self.frame, text=msg)
            self.text.window_create('end', window=self.label)
            self.text.insert('end', '\n')
        # Label
        self.value_list = []
        lf_list = []
        times = 0
        for x in args:
            if type(x[0]) == type(1):
                self.value_list.append(IntVar())
            else:
                self.value_list.append(StringVar())
            # Value
            lf_list.append(LabelFrame(self.frame, text=str(times + 1)))
            self.text.window_create('end', window=lf_list[-1])
            self.text.insert('end', '\n')
            # LabelFrame
            for y in x:
                Radiobutton(lf_list[times], text=y, variable=self.value_list[times], value=y).pack(side='left')
            # Tkinter
            times += 1
        # Checkbutton
        self.text.config(state='disabled')
        # Window
    def config(self, msg=''):
        if msg != '':
            self.label = Label(self.frame, text=msg)
            self.text.window_create('1.0', window=self.label)
            self.text.insert('1.1', '\n')
        else:
            self.label.forget()
    def get(self):
        get = []
        for x in self.value_list:
            get.append(x.get())
        # Get
        point = 0
        for x in range(0, len(get)):
            if get[x] == self.answer[x]:
                point += self.points[x]
        return [get, point]

clear()



# -*- coding: utf-8  -*-

from setuptools import *

setup(
    name = 'Python-Extension',
    version = '1.5.3',
    description = 'Python extension functions',
    license = 'GPL',
    author = 'Yile Wang',
    author_email = 'bluewindde@163.com',
    packages = find_packages(),
    python_requires = '>=2.7',
    include_package_data = True
    )



# -*- coding: utf-8 -*-

raise SystemExit('This module is be used to test')

try:
    import __init__ as tkex
    import blackboard
    import system
    import timer
    import tix.dialog as tktix
    import tix.filedialog as tixfile
    import turtledrawer as tdraw
except (ImportError, ModuleNotFoundError) as msg:
    import tkextension as tkex
    import tkextension.blackboard
    import tkextension.system
    import tkextension.timer
    import tkextension.tix.dialog as tktix
    import tkextension.tix.filedialog as tixfile
    import tkextension.turtledrawer as tdraw
from tkinter import *
import os

def _clear():
    try:
        for root, dirs, files in os.walk('__pycache__', topdown=False):
            for name in files:
                os.remove(os.path.join(root, name))
        os.rmdir('__pycache__')
    except OSError:
        pass
    try:
        for root, dirs, files in os.walk('tix/__pycache__', topdown=False):
            for name in files:
                os.remove(os.path.join(root, name))
        os.rmdir('tix/__pycache__')
    except OSError:
        pass
    try:
        os.remove('word bank.dat')
    except OSError:
        pass
    return True

def _test_tktixtension():
    tktix.askvalue()
    tktix.askitem()
    tktix.askanswer()
    tktix.controlboard()
    tktix.singlechoices()
    tktix.answersheet()
    tk = Tk()
    tktix.create(tk)
    tk.destroy()
    _clear()
    return True

def _test_blackboard():
    return _test_tix_blackboard()

def _test_system():
    system.opensource('tktixtension')
    system.opensource('blackboard')
    system.opensource('test')
    system.opensource('timer')
    system.opensource('tix')
    system.opensource('tix.filedialog')
    system.opensource('turtledrawer')
    _clear()
    return True

def _test_timer():
    return _test_tix_timer()

def _test_tix():
    tk = Tk()
    tk.title('_test_tix')
    # Tkinter
    tkex.AskValue(tk)
    tkex.AskItem(tk)
    tkex.AskAnswer(tk)
    v = IntVar()
    tkex.ControlBoard(tk)
    tkex.SingleChoices(tk)
    tkex.AnswerSheet(tk)
    _clear()
    return True

def _test_tix_blackboard():
    tk = Tk()
    tk.title('_test_tix_blackboard')
    # Tkinter
    x = tk.winfo_screenwidth()
    y = tk.winfo_screenheight()
    blackboard.BlackBoard(tk, width=x - 100, height=y - 250).place(relx=0.5, rely=0.5, relheight=1, relwidth=1, anchor='center')
    tk.mainloop()
    _clear()
    return True

def _test_tix_filedialog():
    tk = Tk()
    tk.title('_test_tix_filedialog')
    # Tkinter
    tixfile.FileTree(tk)
    _clear()
    return True

def _test_tix_timer():
    tk = Tk()
    tk.title('_test_tix_timer')
    # Tkinter
    timer.Timer(tk, 'timer', 1)
    timer.Timer(tk, 'down count', 1)
    timer.Timer(tk, 'alarm clock', 1)
    _clear()
    return True

def _test_turtledrawer():
    a = tdraw.Draw()
    a.create_triangle(1)
    a.create_rectangle(1, 1)
    a.create_pentagon(1)
    a.create_polygon(6, 1)
    a.create_pg(3, 3)
    a.create_pg1(3, 3)
    a.create_koch(1)
    a.create_koch_snowflake(1)
    _clear()
    return True

def _test(module='all'):
    if module == 'all':
        return (
            _test_tkextension(),
            _test_system(),
            _test_tix(),
            _test_tix_blackboard(),
            _test_tix_filedialog(),
            _test_tix_timer(),
            _test_turtledrawer()
            )
    elif module == 'tktixtension':
        return _test_tkexxtension()
    elif module == 'blackboard':
        return _test_blackboard()
    elif module == 'system':
        return _test_system()
    elif module == 'timer':
        return _test_timer()
    elif module == 'tix':
        return _test_tix()
    elif module == 'tix.blackboard':
        return _test_tix_blackboard()
    elif module == 'tix.filedialog':
        return _test_tix_filedialog()
    elif module == 'tix.timer':
        return _test_tix_timer()
    elif module == 'turtledrawer':
        return _test_turtledrawer()

_clear()



# -*- coding: utf-8  -*-

import sys
if sys.version_info[0] != 3 or sys.version_info[1] < 4:
    raise SystemExit('This module needs Python 3.4 or more later')

from tkinter import *
from tkinter.messagebox import *
from tkinter.filedialog import *
from pyextension import *
import tkextension as tix
import platform
import pickle
import time
import os

def askvalue(title='', msg='', args=(0, 100)):
    tk = Tk()
    tk.title(title)
    tk.resizable(0, 0)
    # Tkinter
    frame = tix.AskValue(tk, msg, args)
    frame.pack()
    Button(tk, text='OK', command=tk.quit).pack()
    # Window
    tk.mainloop()
    # Mainloop
    get = frame.get()
    tk.destroy()
    return get
# Askvalue
def askitem(title='', msg='', items=[], number=1, normal=0):
    tk = Tk()
    tk.title(title)
    tk.resizable(0, 0)
    # Tkinter
    frame = tix.AskItem(tk, msg, items, number, normal)
    frame.pack()
    Button(tk, text='OK', command=tk.quit).pack()
    # Window
    tk.mainloop()
    # Mainloop
    get = frame.get()
    tk.destroy()
    return get
# Askitem
def controlboard(title='', msg='', item=[('scale', '', 1, 100), ('listbox', '', [1, 2, 3])]):
    tk = Tk()
    tk.title(title)
    tk.resizable(0, 0)
    # Tkinter
    frame = tix.ControlBoard(tk, msg, item)
    frame.pack()
    Button(tk, text='Finish', command=tk.quit).pack()
    # Window
    tk.mainloop()
    # Mainloop
    get = frame.get()
    tk.destroy()
    return get
# Controlboard
def singlechoices(title='', msg='', args=(('a', 'b', 'c'), ('e', 'f', 'g'))):
    tk = Tk()
    tk.title(title)
    tk.resizable(0, 0)
    # Tkinter
    a = tix.SingleChoices(tk, msg=msg, args=args)
    a.pack()
    Button(tk, text='Finish', command=tk.quit).pack()
    # Window
    tk.mainloop()
    # Mainloop
    get = a.get()
    tk.destroy()
    return get
# Singlechoices
def answersheet(title='', msg='', args=[('a', 'b', 'c')], answer=['a'], points=[1]):
    tk = Tk()
    tk.title(title)
    tk.resizable(0, 0)
    # Tkinter
    a = tix.AnswerSheet(tk, msg=msg, args=args, answer=answer, points=points)
    a.pack()
    Button(tk, text='Finish', command=tk.quit).pack()
    # Window
    tk.mainloop()
    # Mainloop
    get = a.get()
    tk.destroy()
    return get
# Answersheet
def create(tkinter_attribute, mode='menu', args=[('1', '<function>')], AddToTkinter=(False, 'grid', 0, 0, 0, 0, 'center'), note='''Args of option menu only takes strings, likes: arg=['a', 'b', 'c']'''):
    tk = tkinter_attribute
    if mode == 'menu':
        a = Menu(tk)
        for x in args:
            a.add_command(label=x[0], command=x[1])
        if AddToTkinter[0] == True:
            tk.config(menu=a)
        return a
    elif mode == 'option menu':
        v = StringVar()
        a = 'OptionMenu(tk, v, '
        for x in arg:
            a = a + '\'' + x + '\''
            if x != args[-1]:
                a = a + ', '
        a = a + ')'
        b = exec(a)
        if AddToTkinter[0] == True:
            if AddToTkinter[1] == 'pack':
                b.pack()
            elif AddToTkinter[1] == 'grid':
                if AddToTkinter[4] == 0:
                    AddToTkinter[4] = 1
                if AddToTkinter[5] == 0:
                    AddToTkinter[5] = 1
                if AddToTkinter[6] == 'center':
                    AddToTkinter[6] = 'w'
                b.grid(column=AddToTkinter[2], row=AddToTkinter[3],
                       columnspan=AddToTkinter[4], rowspan=AddToTkinter[5],
                       sticky=AddToTkinter[6])
            elif AddToTkinter[1] == 'place':
                if AddToTkinter[4] != 0 and AddToTkinter[5] != 0:
                    b.place(relx=AddToTkinter[2], rely=AddToTkinter[3],
                            relwidth=AddToTkinter[4], relheight=AddToTkinter[5],
                            anchor=AddToTkinter[6])
                elif AddToTkinter[4] == 0 and AddToTkinter[5] == 0:
                    b.place(relx=AddToTkinter[2], rely=AddToTkinter[3],
                            anchor=AddToTkinter[6])
                else:
                    raise ValueError('Both relwidth and relheight must be 0 or not 0')
        return b
# Create



# -*- coding: utf-8  -*-

import sys
if sys.version_info[0] != 3 or sys.version_info[1] < 4:
    raise SystemExit('This module needs Python 3.4 or more later')

from tkinter import *
from tkinter.colorchooser import *
from tkextension.tix.dialog import *
import tkextension as tix

class BlackBoard(tix.Layout):
    def __init__(self, tkinter_attribute, width=400, height=200, bg='#ffffff'):
        self.sqrite_type = 'line' # Sqrite
        self.color = 'black' # Color
        self.width = 1 # R
        self.x = width // 2 # Line
        self.y = height // 2 # Line
        self.on_slide = ['off'] # Slide
        self.do = [] # Undo
        # Value
        self.tk = tkinter_attribute
        self.frame = Frame(self.tk)
        self.canvas = Canvas(self.frame, width=width, height=height, bg=bg)
        self.canvas.pack()
        lb = LabelFrame(self.frame, text='')
        lb.pack()
        self.sqriteset_btn = Button(lb, text='Sqrite', command=self.sqriteset)
        self.sqriteset_btn.grid(column=0, row=0, columnspan=2)
        self.colorchoose_btn = Button(lb, text='A', command=self.colorchoose)
        self.colorchoose_btn.grid(column=0, row=1)
        self.width_btn = Button(lb, text='Width ( W )', command=self.widthset)
        self.width_btn.grid(column=0, row=2)
        self.motion_on_btn = Button(lb, text='Turn on slide painting ( O )', command=self.onslide, state='normal')
        self.motion_on_btn.grid(column=1, row=1)
        self.motion_off_btn = Button(lb, text='Turn off slide painting ( F )', command=self.offslide, state='disabled')
        self.motion_off_btn.grid(column=1, row=2)
        self.undo_btn = Button(lb, text='Undo ( Z )', command=self.undo)
        self.undo_btn.grid(column=0, row=3)
        self.clean_btn = Button(lb, text='Clear', command=self.clean)
        self.clean_btn.grid(column=1, row=3)
        # Window
        self.canvas.bind_all('<KeyRelease>', self.key)
        self.canvas.bind('<Button-2>', self.sqrite_x)
        self.canvas.bind('<ButtonRelease-2>', self.sqrite_y)
        self.canvas.bind('<Button-1>', self.onslide)
        self.canvas.bind('<ButtonRelease-1>', self.offslide)
        self.canvas.bind('<Motion>', self.draw)
        # Bind
    def key(self, event):
        if event.char == 'A' or event.char == 'a':
            self.colorchoose(event)
        elif event.char == 'W' or event.char == 'w':
            self.widthset(event)
        elif event.char == 'O' or event.char == 'o':
            self.onslide(event)
        elif event.char == 'F' or event.char == 'f':
            self.offslide(event)
        elif event.char == 'Z' or event.char == 'z':
            self.undo(event)
        elif event.char == 'C' or event.char == 'c':
            self.clean(event)
    def draw(self, event):
        if self.on_slide[-1] == 'on':
            x1, y1 = (event.x - self.width), (event.y - self.width)
            x2, y2 = (event.x + self.width), (event.y + self.width)
            try:
                self.do[-1].append(self.canvas.create_oval(x1, y1, x2, y2, fill=self.color))
            except IndexError:
                self.do.append([])
                self.do[-1].append(self.canvas.create_oval(x1, y1, x2, y2, fill=self.color))
    def sqriteset(self, event=None):
        self.sqrite_type = askvalue('', 'Sqrite Choose', ['line', 'rectangle', 'oval', 'polygon'])
    def sqrite_x(self, event):
        self.x = event.x
        self.y = event.y
    def sqrite_y(self, event):
        if self.sqrite_type == 'line':
            self.do.append([self.canvas.create_line(self.x, self.y, event.x, event.y, fill=self.color)])
        elif self.sqrite_type == 'rectangle':
            self.do.append([self.canvas.create_rectangle(self.x, self.y, event.x, event.y, fill=self.color)])
        elif self.sqrite_type == 'oval':
            self.do.append([self.canvas.create_oval(self.x, self.y, event.x, event.y, fill=self.color)])
        elif self.sqrite_type == 'polygon':
            self.do.append([self.canvas.create_polygon(self.x, self.y, event.x, event.y, fill=self.color)])
    def onslide(self, event=None):
        self.on_slide.append('on')
        self.motion_on_btn.config(state='disabled')
        self.motion_off_btn.config(state='normal')
        self.do.append([])
    def offslide(self, event=None):
        self.on_slide.append('off')
        self.motion_on_btn.config(state='normal')
        self.motion_off_btn.config(state='disabled')
    def colorchoose(self, event=None):
        self.color = askcolor()
        self.color = self.color[1]
        self.colorchoose_btn.config(text='A', fg=self.color)
    def widthset(self, event=None):
        self.width = int(askanswer('', 'Width'))
    def undo(self, event=None):
        try:
            for x in self.do[-1]:
                self.canvas.delete(x)
            del self.do[-1]
        except IndexError:
            pass
    def clean(self, event=None):
        self.canvas.delete('all')



# -*- coding: utf-8  -*-

import sys
if sys.version_info[0] != 3 or sys.version_info[1] < 4:
    raise SystemExit('This module needs Python 3.4 or more later')

from tkinter import *
from tkinter.messagebox import *
import tkextension as tix
import time

class Timer(tix.Layout):
    def __init__(self, window, mode='timer', arg=0):
        self.end = False
        self.tk = window
        self.mode = mode
        if mode == 'timer':
            self.window = Label(window, text=0)
        elif mode == 'down count':
            self.time = arg
            self.window = Label(window, text=self.time)
        elif mode == 'alarm clock':
            self.time = arg
        else:
            raise AttributeError(''' 'Time' object has no attribute '%s' ''' % mode)
    def timer(per1second):
        psecond = per1second % 100
        # Per 1 S
        second = per1second // 100
        while second >= 60:
            second -= 60
        # S
        minute = per1second // 100 // 60
        while minute >= 60:
            minute -= 60
        # Min
        hour = per1second // 100 // 60 // 60
        while hour >= 24:
            hour -= 24
        # H
        day = per1second // 100 // 60 // 60 // 24
        # D
        return '%s : %s : %s : %s : %s' % (day, hour, minute, second, psecond)
    def mainloop(self):
        if self.mode == 'timer':
            per1second = 0
            while self.end != True:
                self.window.config(text=Timer.timer(per1second))
                self.tk.update()
                time.sleep(0.01)
                per1second += 1
        elif self.mode == 'down count':
            while self.time > 0:
                self.window.config(text=Timer.timer(self.time))
                self.tk.update()
                time.sleep(0.01)
                self.time -= 1
        elif self.mode == 'alarm clock':
            while self.time > 0:
                self.tk.update()
                time.sleep(0.01)
                self.time -= 1
    def stop(self):
        self.end == True



# -*- coding: utf-8  -*-

def floating_point_numbers_retain_decimal_digit(x, decimal_digit=2):
    dd = decimal_digit
    a = 10 ** dd * x
    a = int(a)
    a /= 10 ** dd
    return a
def floatdd(x, decimal_digit=2):
    return floating_point_numbers_retain_decimal_digit(x, decimal_digit)

def fibo(num=None, found=None, to=None):
    if num != None:
        fib = [1]
        for x in range(0, num):
            try:
                fib[-2]
            except IndexError:
                fib.append(1)
                continue
            fib.append(fib[-1] + fib[-2])
        return fib[num]
    elif found != None:
        fib = [1]
        if type(found) != type(1):
            raise TypeError('\'found\' must be int')
        while True:
            if fib[-1] > found:
                fib.append(False)
                break
            elif fib[-1] == found:
                fib.append(True)
                break
            try:
                fib[-2]
            except IndexError:
                fib.append(1)
                continue
            fib.append(fib[-1] + fib[-2])
        return fib[-1]
    elif to != None:
        fib = [1]
        for x in range(0, to):
            try:
                fib[-2]
            except IndexError:
                fib.append(1)
                continue
            fib.append(fib[-1] + fib[-2])
        return fib

def arithmetic_sequence(result_type, sequence=None, first=None, last=None, number_of_items=None, common_difference=None):
    if result_type == 'all':
        if type(sequence) == type(''):
            lst = []
            for x in sequence:
                lst.append(int(x))
            s = sum(lst)
            cd = int(sequence[1]) - int(sequence[0])
            return {
                'sequence' : sequence,
                'first' : int(sequence[0]),
                'last' : int(sequence[-1]),
                'number of items' : len(sequence),
                'common difference' : cd,
                'sum' : s
                }
        elif type(sequence) == type([]):
            cd = sequence[1] - sequence[0]
            return {
                'sequence' : sequence,
                'first' : sequence[0],
                'last' : sequence[-1],
                'number of items' : len(sequence),
                'common difference' : cd,
                'sum' : sum(sequence)
                }
        else:
            raise TypeError('\'sequence\' must be a list type or a string type')
    elif result_type == 'output':
        if first != None:
            output = [first]
            for x in range(1, number_of_items):
                output.append(output[-1] + common_difference)
        else:
            output = [last]
            for x in range(1, number_of_items):
                output.append(output[-1] - common_difference)
            output = output[::-1]
        return output
    elif result_type == 'first':
        return arithmetic_sequence('output', first, last, number_of_items, common_difference)[0]
    elif result_type == 'last':
        return arithmetic_sequence('output', first, last, number_of_items, common_difference)[-1]
    elif result_type == 'number_of_items' or result_type == 'number of items':
        output = [first]
        while output[-1] < last:
            output.append(output[-1] + common_difference)
        return len(output)
    elif result_type == 'common_difference' or result_type == 'common difference':
        return arithmetic_sequence('output', first, last, number_of_items, common_difference)[1] - arithmetic_sequence('output', first, last, number_of_items, common_difference)[0]
    elif result_type == 'sum':
        return sum(arithmetic_sequence('output', first, last, number_of_items, common_difference))

from math import *
import cmath as c

def math(mode='+', a=0, b=0):
    if a == 'pi':
        a = pi
    if b == 'pi':
        b = pi
    # pi
    if a == 'e':
        a = e
    if b == 'e':
        b = e
    # e
    if a == 'tau':
        a = tau
    if b == 'tau':
        b = tau
    # tau
    if a == 'inf':
        a = float('inf')
    if b == 'inf':
        b = float('inf')
    # inf
    if a == '-inf':
        a = float('-inf')
    if b == '-inf':
        b = float('-inf')
    # -inf
    if a == 'nan' or a == 'NaN':
        a = nan
    if b == 'nan' or b == 'NaN':
        b = nan
    # nan
    if mode == '+':
        result = a + b
    elif mode == '-':
        result = a - b
    elif mode == '*':
        result = a * b
    elif mode == '/':
        result = a / b
    elif mode == '//':
        result = a // b
    elif mode == '...':
        result = a % b
    elif mode == '%':
        result = a % b
    # Operations
    elif mode == '>':
        if a > b:
            result = True
        else:
            result = False
    elif mode == '<':
        if a < b:
            result = True
        else:
            result = False
    elif mode == '==':
        if a == b:
            result = True
        else:
            result = False
    elif mode == '!=':
        if a != b:
            result = True
        else:
            result = False
    elif mode == '>=':
        if a >= b:
            result = True
        else:
            result = False
    elif mode == '<=':
        if a <= b:
            result = True
        else:
            result = False
        # Compare
    elif mode == '!':
        x = 1
        for y in range(1, a + 1):
            x = x * y
        result = x
    elif mode == '^':
        result = a ** b
    elif mode == 'square root':
        result = sqrt(a)
    elif mode == 'sqrt()':
        result = sqrt(a)
    # Power
    elif mode == 'exp()':
        result = exp(a)
    elif mode == 'exp1()':
        result = exp1(a)
    # Power And Logarithmic Functions
    elif mode == 'cos()':
        result = cos(a)
    elif mode == 'tan()':
        result = tan(a)
    elif mode == 'sin()':
        result = sin(a)
    elif mode == 'acos()':
        result = acos(a)
    elif mode == 'atan()':
        result = atan(a)
    elif mode == 'asin()':
        result = asin(a)
    elif mode == 'dist()':
        result = dist(a)
    elif mode == 'hypot()':
        result = hypot(a)
    # Trigonometric Function
    elif mode == 'cosh()':
        result = cosh(a)
    elif mode == 'tanh()':
        result = tanh(a)
    elif mode == 'sinh()':
        result = sinh(a)
    elif mode == 'acosh()':
        result = acosh(a)
    elif mode == 'atanh()':
        result = atanh(a)
    elif mode == 'asinh()':
        result = asinh(a)
    # Hyperbolic Functions
    elif mode == 'Degrees()':
        result = degrees(a)
    elif mode == 'degrees()':
        result = degrees(a)
    elif mode == 'Radians()':
        result = radians(a)
    elif mode == 'radians()':
        result = radians(a)
    # Angular Conversion
    elif mode == 'abs()':
        result = abs(a)
        # Else
    elif mode == 'Int / Float':
        if type(a) == type(1):
            result = float(a)
        elif type(a) == type(1.5):
            result = int(a)
        else:
            raise TypeError(''' 'Int / Float' object's argument must be int or float ''')
    elif mode == 'int / float':
        if type(a) == type(1):
            result = float(a)
        elif type(a) == type(1.5):
            result = int(a)
        else:
            raise TypeError(''' 'Int / Float' object's argument must be int or float ''')
    elif mode == 'Type':
        result = type(a)
    elif mode == 'type':
        result = type(a)
    # Type
    else:
        raise AttributeError(''' 'run' object has no attribute '%s' ''' % mode)
    return result

def cmath(mode, a, b):
    if a == 'pi':
        a = c.pi
    if b == 'pi':
        b = c.pi
    # pi
    if a == 'e':
        a = c.e
    if b == 'e':
        b = c.e
    # e
    if a == 'tau':
        a = c.tau
    if b == 'tau':
        b = c.tau
    # tau
    if a == 'inf':
        a = float('inf')
    if b == 'inf':
        b = float('inf')
    # inf
    if a == '-inf':
        a = float('-inf')
    if b == '-inf':
        b = float('-inf')
    # -inf
    if a == 'nan' or a == 'NaN':
        a = c.nan
    if b == 'nan' or b == 'NaN':
        b = c.nan
    # nan
    if mode == 'cos()':
            return c.cos(a)
    elif mode == 'tan()':
            return c.tan(a)
    elif mode == 'sin()':
            return c.sin(a)
    elif mode == 'acos()':
            return c.cos(a)
    elif mode == 'atan()':
            return c.tan(a)
    elif mode == 'asin()':
            return c.sin(a)
        # Trigonometric Function
    elif mode == 'cosh()':
            return c.cos(a)
    elif mode == 'tanh()':
            return c.tan(a)
    elif mode == 'sinh()':
            return c.sin(a)
    elif mode == 'acosh()':
            return c.cos(a)
    elif mode == 'atanh()':
            return c.tan(a)
    elif mode == 'asinh()':
            return c.sin(a)
        # Hyperbolic Functions
    else:
            raise AttributeError(''' 'crun' object has no attribute '%s' ''' % mode)

def meaning(mode=''):
    if mode == '+':
        message = 'Find the sum of A and B, indicating A plus B'
    elif mode == '-':
        message = 'Find the difference between A and B, which means A minus B'
    elif mode == '*':
        message = 'Find the product of A and B, representing A times B'
    elif mode == '/':
        message = 'The quotient of A and B, which means A divided by B'
    elif mode == '//':
        message = 'Find the integer part of a divided by B'
    elif mode == '...':
        message = 'Find the remainder of a divided by B'
    elif mode == '%':
        message = 'Find the remainder of a divided by B'
        # Operations
    elif mode == '>':
        message = 'Find whether A is greater than B. if yes, return True; If not, False is returned'
    elif mode == '<':
        message = 'Find whether A is less than B. If yes, return true; If not, false is returned'
    elif mode == '==':
        message = 'Find whether A is equal to B. If yes, return true; If not, false is returned'
    elif mode == '>=':
        message = 'Find whether A is greater than or equal to B. If yes, return true; If not, false is returned'
    elif mode == '<=':
        message = 'Find whether A is less than or equal to B. If yes, return true; If not, false is returned'
    elif mode == '!=':
        message = 'Find whether A is not equal to B. If yes, return true; If not, false is returned'
        # Compare
    elif mode == '!':
        message = 'Find the factorial of A'
    if mode == '^':
        message = 'Seeking the b-th of a'
    if mode == 'sqrt()':
        message = 'Find the square root of A'
        # Power
    elif mode == 'cos()':
        message = 'Return the cosine of x radians'
    elif mode == 'sin()':
        message = 'Return the sine of x radians'
    elif mode == 'tan()':
        message = 'Return the tangent of x radians'
    elif mode == 'acos()':
        message = 'Return the arc cosine of A, in radians'
    elif mode == 'asin()':
        message = 'Return the arc sine of A, in radians'
    elif mode == 'atan()':
        message = 'Return the arc tangent of A, in radians'
    elif mode == 'dist()':
        message = 'Return the Euclidean distance between two points A and B, \n each given as a sequence (or iterable) of coordinates. \n The two points must have the same dimension'
    elif mode == 'hypot()':
        message = '''Return the Euclidean norm,
sqrt(sum(A**2 for A in coordinates)).
This is the length of the vector from the origin to the point given by the coordinates.
For a two dimensional point (A, B),
this is equivalent to computing the hypotenuse of a right triangle using the Pythagorean theorem, sqrt(A*A + B*B)'''
        # Trigonometric Function
    elif mode == 'acosh()':
        message = 'Return the inverse hyperbolic cosine of A'
    elif mode == 'asinh()':
        message = 'Return the inverse hyperbolic sine of A'
    elif mode == 'atanh()':
        message = 'Return the inverse hyperbolic tangent of A'
    elif mode == 'cosh()':
        message = 'Return the hyperbolic cosine of A'
    elif mode == 'sinh()':
        message = 'Return the hyperbolic sine of A'
    elif mode == 'tanh()':
        message = 'Return the hyperbolic tangent of A'
        # Hyperbolic Functions
    elif mode == 'exp()':
        message = 'Return e raised to the power A, where e = 2.718281… is the base of natural logarithms. \n This is usually more accurate than math.e ** A or pow(math.e, A)'
    elif mode == 'expm1()':
        message = '''Return e raised to the power A, minus 1.
Here e is the base of natural logarithms.
For small floats A, the subtraction in exp(A) - 1 can result in a significant loss of precision;
the expm1() function provides a way to compute this quantity to full precision:

>>> from math import exp, expm1
>>> exp(1e-5) - 1  # gives result accurate to 11 places
1.0000050000069649e-05
>>> expm1(1e-5)    # result accurate to full precision
1.0000050000166668e-05'''
        # Power And Logarithmic Functions
    elif mode == 'Degrees()':
        message = 'Convert angle A from radians to degrees'
    elif mode == 'degrees()':
        message = 'Convert angle A from radians to degrees'
    elif mode == 'Radians()':
        message = 'Convert angle A from degrees to radians'
    elif mode == 'radians()':
        message = 'Convert angle A from degrees to radians'
        # Angular Conversion
    elif mode == 'abs()':
        message = 'Return the absolute value of a'
        # Else
    elif mode == 'Int / Float':
        message = 'Force integer A to be converted to decimal, or convert decimal A to integer ( Note: if the decimal part of A is 0, A will be converted to decimal x.0 )'
    elif mode == 'int / float':
        message = 'Force integer A to be converted to decimal, or convert decimal A to integer ( Note: if the decimal part of A is 0, A will be converted to decimal x.0 )'
    elif mode == 'Type':
        message = 'Returns the type of A, integer returns Int, decimal returns Float \n ( Note: if decimal part is 0, Int is returned )'
    elif mode == 'type':
        message = 'Returns the type of A, integer returns Int, decimal returns Float \n ( Note: if decimal part is 0, Int is returned )'
        # Type
    else:
            raise AttributeError(''' 'meaning' object has no attribute '%s' ''' % mode)

def list():
    a = '''
+, -, *, /, //, ..., %,
^, sqrt(), sqare root,
exp(), exp1(),
cos(), tan(), sin(),
acos(), atan(), asin(),
cosh(), tanh(), sinh(),
acosh(), atanh(), asinh(),
abs(),
Int / Float, Type
'''
    return a
